/**
 * @license
 * Copyright © 2017 Intuilab
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 *
 * The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability,
 * fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability,
 * whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
 *
 * Except as contained in this notice, the name of Intuilab shall not be used in advertising or otherwise to promote the sale,
 * use or other dealings in this Software without prior written authorization from Intuilab.
 */

WebTriggers.prototype = new EventEmitter();        // Here's where the inheritance occurs
WebTriggers.prototype.constructor = WebTriggers;





/**
 * @constructor
 */
function WebTriggers() {

    var self = this;

    this.instanceKey = "IA_WebTriggers";
    this.InterfaceAssetID = this._guid();

    this.WTLog("Start IA instance", null);

    // events that can be received here (so sent by Server)
    this.RequestEvent = {
        getInfo : 'getInfo', message: 'message', ping: 'pingIntuiFace'
    };


    // events that can be sent from here (so received by Server)
    this.ResponseEvent = {
        message: 'message', getInfoResponse: 'getInfoResponse', pong: 'pongIntuiFace'
    };

    this.isConnected = false;

    // Http requests management
    this.httpService = intuiface.get("httpService");
    this.playerService = intuiface.get("playerService");
    this.socketIoService = intuiface.get("socketIoService");

    this.webTriggersBasePath = this.playerService.generateIntuiFaceWebTriggersUrl();
    this.WTLog("WebSocket Server URL: " + this.webTriggersBasePath, null);

    this.tags = [];
    this.userCredentialKeys = [];

    this.deviceInfo = this._getDeviceInfo();

    this.initSocket();

}


WebTriggers.prototype._guid = function () {
    function s4() {
        return Math.floor((1 + Math.random()) * 0x10000)
            .toString(16)
            .substring(1);
    }
    return s4() + s4() + '-' + s4() + '-' + s4() + '-' + s4() + '-' + s4() + s4() + s4();
};

WebTriggers.prototype.initSocket = function () {
    var self = this;

    if (this.playerService.IsInDesignMode()){
        this.WTLog("We are in Composer Design mode, so we don't start Web Socket connection", null);
        return;
    }

    this.WTLog("We are in Composer Play mode or on Player, so we start to init Web Socket connection", null);
    this.socketIoService.openWebSocket(this.webTriggersBasePath, null, {
        "connect": function (result) {
            self.WTLog(" //// Receive EVENT ////: connect to: " + self.webTriggersBasePath, null);
            //self.setIsConnected(true);
        },
        "registeredDevice": function (result) {
            self.WTLog(" //// Receive EVENT ////: registeredDevice", null);
            self.setIsConnected(true);
        },
        "disconnect": function (data) {
            self.WTLog(" //// Receive EVENT ////: disconnect", null);
            self.setIsConnected(false);
        },
        "message": function (data) {
            self.WTLog(" //// Receive EVENT ////: message", data);
        },
        "error": function (error) {
            self.WTLog(" //// Receive EVENT ////: error", error);
        },
        "connect_error": function (error) {
            self.WTLog(" //// Receive EVENT ////: connect_error, unable to connect to: " + self.webTriggersBasePath, error);
        },
        "connect_timeout": function (error) {
            self.WTLog(" //// Receive EVENT ////: connect_timeout, unable to connect to: " + self.webTriggersBasePath, error);
        },
        "reconnect": function (error) {
            self.WTLog(" //// Receive EVENT ////: reconnect, to: " + self.webTriggersBasePath, error);
        },
        "reconnecting": function (error) {
            self.WTLog(" //// Receive EVENT ////: reconnecting, to: " + self.webTriggersBasePath, error);
        },
        "reconnect_attempt": function (error) {
            self.WTLog(" //// Receive EVENT ////: reconnect_attempt, unable to connect to: " + self.webTriggersBasePath, error);
        },
        "reconnect_error": function (error) {
            self.WTLog(" //// Receive EVENT ////: reconnect_error, unable to connect to: " + self.webTriggersBasePath, error);
        },
        "reconnect_failed": function (error) {
            self.WTLog(" //// Receive EVENT ////: reconnect_failed, unable to connect to: " + self.webTriggersBasePath, error);
        },
        "pingIntuiFace": function (error) {
            self.WTLog(" //// Receive EVENT ////: ping", error);
            self._emitPong("");
        },
        "getInfo": function (data) {
            self.WTLog(" //// Receive EVENT ////: getInfo (Web Socket server ask an identification to this IA", null);
            self._sendInfoResponse(data);
        },
        "sendMessage": function (data) {
            self.WTLog(" //// Receive EVENT ////: sendMessage", null);
            self._externalMessage(data);
        }},
        this.instanceKey,
        this.InterfaceAssetID,
        function (isConnected) {
            if ((isConnected != null) && (typeof isConnected !== 'object')) {
                isConnected = JSON.parse(isConnected);
            }
            self.WTLog("Connection status (used when there are multiple instance of this Web triggers IA [IsConnected: " + isConnected + "]" , null);
            //self.setIsConnected(isConnected);
        }
        );

};


WebTriggers.prototype.setIsConnected = function (isConnected) {
    var self = this;

    if (this.isConnected != isConnected) {

        this.isConnected = isConnected;
        if (this.isConnected){
            this.WTLog("Web Socket is connected", null);
        }
        else{
            this.WTLog("Web Socket is no more connected", null);
        }
        this.emit('isConnectedChanged');

        if (isConnected){
            this.WTLog("Fire the \"connected\" Trigger", null);
            this.emit('connected');
        }
        else{
            this.WTLog("Fire the \"disconnected\" Trigger", null);
            this.emit('disconnected');
        }

    }
};

WebTriggers.prototype._getDeviceInfo = function() {

    var self = this;

    var licenseId = this.playerService.getLicenseId();

    var deviceInfo = {
        OS : this.playerService.getPlatform(),
        OSVersion : this.playerService.getOSVersion(),
        applicationName : this.playerService.getApplicationName(),
        completeApplicationName : this.playerService.getCompleteApplicationName(),
        intuifaceVersion : this.playerService.getIntuiFaceVersion(),
        intuifaceLicense : licenseId,
        playerId : this.playerService.getDeviceGuid(),
        deviceName : this.playerService.getDeviceName(),
        deviceNickName : this.playerService.getDisplayName(),
        experienceName: this.playerService.getCurrentExperienceName(),
        experienceId: this.playerService.getCurrentExperienceId(),
        customerEmail : this.playerService.getCustomerEmail()//,
        //credentialKeys : credentialKeys,
        //tags : this.tags
    };

    this.playerId = deviceInfo.playerId;
    this.WTLog("The player ID (GUID) is: " + this.playerId, null);
    this.emit('playerIdChanged');

    this.deviceName = deviceInfo.deviceNickName;
    this.WTLog("The device name (deviceNickName) is: " + this.deviceName, null);
    this.emit('deviceNameChanged');

    this.experienceName = deviceInfo.experienceName;
    this.WTLog("The experience name is: " + this.experienceName, null);
    this.emit('experienceNameChanged');

    this.experienceId = deviceInfo.experienceId;
    this.WTLog("The experience Id is: " + this.experienceId, null);
    this.emit('experienceIdChanged');

    this.WTLog("Get all device Info completed", null);
    return deviceInfo;

};

WebTriggers.prototype._sendInfoResponse = function(data) {
    var self = this;

    this.WTLog("Start to get User Details", null);
    this.playerService.getUserDetails(function (result) {self.getUserDetailsSuccess(result)}, function (result) {self.getUserDetailsError(result)});

};

WebTriggers.prototype.getUserDetailsSuccess = function (userDetails) {
    var self = this;
    var credentialKeys = [];

    if (typeof userDetails !== 'object') {
        userDetails = JSON.parse(userDetails);
    }
    if (userDetails.CredentialScopes){
        for (var indexScope in userDetails.CredentialScopes){
            var credentialScope = userDetails.CredentialScopes[indexScope];
            if (credentialScope.Name === "WebTriggers"){
                this.WTLog("We found WebTriggers credential key scope", null);
                for (var indexKey in credentialScope.CredentialKeys){
                    var credentialKey = credentialScope.CredentialKeys[indexKey];
                    if (credentialKey.KeyId){
                        credentialKeys.push(credentialKey.KeyId);
                        this.WTLog("Use credential key: " + credentialKey.KeyId.substring(0, 5) + "...", null);
                    }
                }
            }
        }
    }
    else{
        this.WTLog("No Credential Scopes found", null);
    }
    if (credentialKeys.length === 0){
        this.WTLog("No WebTriggers credential key scope found !!!", null);
    }

    if (typeof credentialKeys !== 'object') {
        credentialKeys = JSON.parse(credentialKeys);
    }
    this.deviceInfo.credentialKeys = credentialKeys;

    if (userDetails.Tags){
        this.tags = userDetails.Tags;
        this.WTLog("We found Player Tag(s): ", this.tags);
    }

    if (typeof this.tags !== 'object') {
        this.tags = JSON.parse(this.tags);
    }
    this.emit('tagsChanged');
    this.deviceInfo.tags = this.tags;
    this.deviceInfo.tagsAsText = this.tags.join(',');

    this.WTLog("Get User Details completed", null);

    this._emitInfoResponse(this.deviceInfo);

};

WebTriggers.prototype.getUserDetailsError = function (error) {

    this.WTLog("WARNING: the state Get User Details Failed: " + error, null);
    this._emitInfoResponse(this.deviceInfo);
};


WebTriggers.prototype._emitInfoResponse = function(data) {
    var self = this;

    self.WTLog("The IA send the identification Info to the Web Triggers Server", null);
    this.socketIoService.emitIo(this.ResponseEvent.getInfoResponse, data, {
        "success": function (result) {
            self.WTLog("Send the identification Info to the Web Triggers Server is completed", null);
        },
        "error": function (errorMessage) {
            self.WTLog("WARNING: the state Send the identification Info to the Web Triggers Server is not completed", errorMessage);
        }},
        this.instanceKey,
        this.InterfaceAssetID);

};


WebTriggers.prototype._emitPong = function(data) {
    var self = this;

    self.WTLog("The IA send the Pong event to the Web Triggers Server", null);
    this.socketIoService.emitIo(this.ResponseEvent.pong, data, {
            "success": function (result) {
                self.WTLog("Send the Pong event to the Web Triggers Server is completed", null);
            },
            "error": function (errorMessage) {
                self.WTLog("WARNING: the state Send the Pong event to the Web Triggers Server is not completed", errorMessage);
            }},
        this.instanceKey,
        this.InterfaceAssetID);

};


WebTriggers.prototype._externalMessage = function(data) {
    this.WTLog("An external message is received", data);
    if (data){
        var objectData = JSON.parse(data);
        if (objectData["msg"] === undefined) objectData["msg"] = "";
        if (objectData["p1"] === undefined) objectData["p1"] = "";
        if (objectData["p2"] === undefined) objectData["p2"] = "";
        if (objectData["p3"] === undefined) objectData["p3"] = "";
        this.WTLog("Fire the \"External message is received\" Trigger with Parameters: message=" + objectData["msg"] + " | parameter1=" + objectData["p1"] + " | parameter2=" + objectData["p2"] + " | parameter3=" + objectData["p3"], null);
        this.emit('externalMsgReceived', [objectData["msg"], objectData["p1"], objectData["p2"], objectData["p3"]]);
    }

};




WebTriggers.prototype._localRemote = function(message) {
    var self = this;
    this.httpService.get("http://localhost:8000/intuifacepresentationplayer/presentation/" + message, {
        "success": function(result) {
            self.WTLog("success localRemote " + message, result);
        },
        "error" : function(errorMessage) {
            self.WTLog("error localRemote " + message, errorMessage);
        }});

};


WebTriggers.prototype._externalTrigger = function(message) {
    var self = this;
    this.httpService.get("http://localhost:8000/intuifacepresentationplayer/input/" + message, {
        "success": function(result) {
            self.WTLog("success externalTrigger " + message, result);
        },
        "error" : function(errorMessage) {
            self.WTLog("error externalTrigger " + message, errorMessage);
        }});

};


WebTriggers.prototype.WTLog = function(text, obj) {
    console.log("------> IA Web Triggers <------ [ID:" + this.InterfaceAssetID.substring(0, 5) + "...] " +  text);
    if (obj){
        console.log("------> IA Web Triggers <------ [ID:" + this.InterfaceAssetID.substring(0, 5) + "...] *** Object start ***");
        console.log(obj);
        console.log("------> IA Web Triggers <------ [ID:" + this.InterfaceAssetID.substring(0, 5) + "...] ***  Object end  ***");
    }
};

/**
 *
 */
WebTriggers.prototype.dispose = function() {
    this.WTLog("Dispose IA", null);
    this.socketIoService.disconnect(this.instanceKey, this.InterfaceAssetID);
};



